@@ -46,7 +46,7 @@ bin_clone(char *nam, char **args, char *ops, int func)
     int ttyfd, pid;
 
     unmetafy(*args, NULL);
-    ttyfd = open(*args, O_RDWR);
+    ttyfd = open(*args, O_RDWR|O_NOCTTY);
     if (ttyfd < 0) {
 	zwarnnam(nam, "%s: %e", *args, errno);
 	return 1;
