@@ -326,9 +326,9 @@ bin_rm(char *nam, char **args, char *ops, int func)
     ds.dirname = NULL;
     ds.dirfd = ds.level = -1;
     if (ops['r'] || ops['s']) {
-	if ((ds.dirfd = open(".", O_RDONLY)) < 0 &&
+	if ((ds.dirfd = open(".", O_RDONLY|O_NOCTTY)) < 0 &&
 	    zgetdir(&ds) && *ds.dirname != '/')
-	    ds.dirfd = open("..", O_RDONLY);
+	    ds.dirfd = open("..", O_RDONLY|O_NOCTTY);
     }
     for(; !errflag && !(err & 2) && *args; args++) {
 	rp = ztrdup(*args);
