@@ -50,7 +50,13 @@ statmodeprint(mode_t mode, char *outbuf, int flags)
 	    strcat(outbuf, " (");
     }
     if (flags & STF_STRING) {
-	char pm[11] = "?---------";
+	static const char *modes = "?rwxrwxrwx";
+	static const mode_t mflags[] = { S_IRUSR, S_IWUSR, S_IXUSR,
+					 S_IRGRP, S_IWGRP, S_IXGRP,
+					 S_IROTH, S_IWOTH, S_IXOTH };
+	const mode_t *mfp = mflags;
+	char pm[11];
+	int i;
 
 	if (S_ISBLK(mode))
 	    *pm = 'b';
@@ -74,25 +80,12 @@ statmodeprint(mode_t mode, char *outbuf, int flags)
 	    *pm = '-';
 	else if (S_ISSOCK(mode))
 	    *pm = 's';
+	else
+	    *pm = '?';
+
+	for (i = 1; i <= 9; i++)
+	    pm[i] = (mode & *mfp++) ? modes[i] : '-';
 
-	if(mode & S_IRUSR)
-	    pm[1] = 'r';
-	if(mode & S_IWUSR)
-	    pm[2] = 'w';
-	if(mode & S_IXUSR)
-	    pm[3] = 'x';
-	if(mode & S_IRGRP)
-	    pm[4] = 'r';
-	if(mode & S_IWGRP)
-	    pm[5] = 'w';
-	if(mode & S_IXGRP)
-	    pm[6] = 'x';
-	if(mode & S_IROTH)
-	    pm[7] = 'r';
-	if(mode & S_IWOTH)
-	    pm[8] = 'w';
-	if(mode & S_IXOTH)
-	    pm[9] = 'x';
 	if (mode & S_ISUID)
 	    pm[3] = (mode & S_IXUSR) ? 's' : 'S';
 	if (mode & S_ISGID)
