@@ -1,6 +1,4 @@
 /*
- * $Id: zle_hist.c,v 3.1.2.5 1997/06/01 06:13:15 hzoli Exp $
- *
  * zle_hist.c - history editing
  *
  * This file is part of zsh, the Z shell.
@@ -29,11 +27,31 @@
  *
  */
 
-#define IN_ZLE
-#define IN_COMP
-#include "zle.h"
+#include "zle.mdh"
 #include "zle_hist.pro"
 
+/* Are references to earlier history lines permitted?  == 0 if       *
+ * editing or reading a standalone line, such as in vared or select. */
+
+/**/
+int histallowed;
+
+/* Column position of vi ideal cursor.  -1 if it is unknown -- most *
+ * movements and changes do this.                                   */
+
+/**/
+int lastcol;
+
+/* current history line number */
+
+/**/
+int histline;
+
+/* the last line in the history (the current one), metafied */
+
+/**/
+char *curhistline;
+
 /**/
 void
 remember_edits(void)
@@ -954,6 +972,11 @@ vifetchhistory(void)
 	feep();
 }
 
+/* the last vi search */
+
+static char *visrchstr;
+static int visrchsense;
+
 /**/
 static int
 getvisrchstr(void)
