@@ -1174,7 +1174,8 @@ bin_fc(char *nam, char **argv, char *ops, int func)
 
 	retval = 1;
 	fil = gettempname();
-	if (((tempfd = open(fil, O_WRONLY | O_CREAT | O_EXCL, 0600)) == -1) ||
+	if (((tempfd = open(fil, O_WRONLY | O_CREAT | O_EXCL | O_NOCTTY, 0600))
+	    == -1) ||
 		((out = fdopen(tempfd, "w")) == NULL)) {
 	    zwarnnam("fc", "can't open temp file: %e", NULL, errno);
 	} else {
@@ -2874,7 +2875,7 @@ int
 bin_read(char *name, char **args, char *ops, int func)
 {
     char *reply, *readpmpt;
-    int bsiz, c = 0, gotnl = 0, al = 0, first, nchars = 1;
+    int bsiz, c = 0, gotnl = 0, al = 0, first, nchars = 1, bslash;
     int haso = 0;	/* true if /dev/tty has been opened specially */
     int isem = !strcmp(term, "emacs");
     char *buf, *bptr, *firstarg, *zbuforig;
@@ -2903,7 +2904,7 @@ bin_read(char *name, char **args, char *ops, int func)
     if ((ops['k'] && !ops['u'] && !ops['p']) || ops['q']) {
 	if (SHTTY == -1) {
 	    /* need to open /dev/tty specially */
-	    SHTTY = open("/dev/tty", O_RDWR);
+	    SHTTY = open("/dev/tty", O_RDWR|O_NOCTTY);
 	    haso = 1;
 	}
 	/* We should have a SHTTY opened by now. */
@@ -2923,27 +2924,12 @@ bin_read(char *name, char **args, char *ops, int func)
     } else if (ops['u'] && !ops['p']) {
 	/* -u means take input from the specified file descriptor. *
 	 * -up means take input from the coprocess.                */
-	for (readfd = 0; readfd < 10; ++readfd)
-	    if (ops[readfd + '0'])
-		break;
-	if (readfd == 10)
-	    readfd = 0;
+	for (readfd = 9; readfd && !ops[readfd + '0']; --readfd);
     } else if (ops['p'])
 	readfd = coprocin;
-    else {
-	/* last resort: take input from plain old stdin */
-	attachtty((jobtab[thisjob].gleader) ? jobtab[thisjob].gleader : mypgrp);
+    else
 	readfd = 0;
-#if 0
-	  else if (isset(SHINSTDIN) && unset(INTERACTIVE)) {
-	    /* use stdout or stderr, if either is a tty */
-	    if (isatty(1))
-		readfd = 1;
-	    else if (isatty(2))
-		readfd = 2;
-	}
-#endif
-    }
+
     /* handle prompt */
     if (firstarg) {
 	for (readpmpt = firstarg;
@@ -2963,7 +2949,7 @@ bin_read(char *name, char **args, char *ops, int func)
 	char d;
 
 	/* allocate buffer space for result */
-	bptr = buf = (char *)zalloc(nchars);
+	bptr = buf = (char *)zalloc(nchars+1);
 
 	do {
 	    /* If read returns 0, is end of file */
@@ -2994,7 +2980,7 @@ bin_read(char *name, char **args, char *ops, int func)
 	if (!ops['e'])
 	    setsparam(reply, metafy(buf, bptr - buf, META_REALLOC));
 	else
-	    zfree(buf, bptr - buf);
+	    zfree(buf, bptr - buf + 1);
 	return val <= 0;
     }
 
@@ -3030,22 +3016,21 @@ bin_read(char *name, char **args, char *ops, int func)
     zbuforig = zbuf = (!ops['z']) ? NULL :
 	(nonempty(bufstack)) ? (char *) getlinknode(bufstack) : ztrdup("");
     first = 1;
+    bslash = 0;
     while (*args || (ops['A'] && !gotnl)) {
 	buf = bptr = (char *)zalloc(bsiz = 64);
 	/* get input, a character at a time */
-	for (;;) {
-	    if (gotnl)
-		break;
+	while (!gotnl) {
 	    c = zread();
 	    /* \ at the end of a line indicates a continuation *
 	     * line, except in raw mode (-r option)            */
-	    if (!ops['r'] && c == '\n' && bptr != buf && bptr[-1] == '\\') {
-		bptr--;
+	    if (bslash && c == '\n') {
+		bslash = 0;
 		continue;
 	    }
 	    if (c == EOF || c == '\n')
 		break;
-	    if (isep(c)) {
+	    if (!bslash && isep(c)) {
 		if (bptr != buf || (!iwsep(c) && first)) {
 		    first |= !iwsep(c);
 		    break;
@@ -3053,6 +3038,9 @@ bin_read(char *name, char **args, char *ops, int func)
 		first |= !iwsep(c);
 		continue;
 	    }
+	    bslash = c == '\\' && !bslash && !ops['r'];
+	    if (bslash)
+		continue;
 	    first = 0;
 	    if (imeta(c)) {
 		*bptr++ = Meta;
@@ -3120,24 +3108,28 @@ bin_read(char *name, char **args, char *ops, int func)
     }
     buf = bptr = (char *)zalloc(bsiz = 64);
     /* any remaining part of the line goes into one parameter */
+    bslash = 0;
     if (!gotnl)
 	for (;;) {
 	    c = zread();
 	    /* \ at the end of a line introduces a continuation line, except in
 	    raw mode (-r option) */
-	    if (!ops['r'] && c == '\n' && bptr != buf && bptr[-1] == '\\') {
-		bptr--;
+	    if (bslash && c == '\n') {
+		bslash = 0;
 		continue;
 	    }
 	    if (c == EOF || (c == '\n' && !zbuf))
 		break;
-	    if (isep(c) && bptr == buf)
+	    if (!bslash && isep(c) && bptr == buf)
 		if (iwsep(c))
 		    continue;
 		else if (!first) {
 		    first = 1;
 		    continue;
 		}
+	    bslash = c == '\\' && !bslash && !ops['r'];
+	    if (bslash)
+		continue;
 	    if (imeta(c)) {
 		*bptr++ = Meta;
 		*bptr++ = c ^ 32;
