@@ -242,7 +242,7 @@ zchdir(char *dir)
 	    break;
 #ifdef HAVE_FCHDIR
 	if (currdir == -2)
-	    currdir = open(".", O_RDONLY);
+	    currdir = open(".", O_RDONLY|O_NOCTTY);
 #endif
 	*s = '\0';
 	if (chdir(dir)) {
