@@ -266,7 +266,7 @@ zexecve(char *pth, char **argv)
 	char execvebuf[POUNDBANGLIMIT + 1], *ptr, *ptr2, *argv0;
 	int fd, ct, t0;
 
-	if ((fd = open(pth, O_RDONLY)) >= 0) {
+	if ((fd = open(pth, O_RDONLY|O_NOCTTY)) >= 0) {
 	    argv0 = *argv;
 	    *argv = pth;
 	    ct = read(fd, execvebuf, POUNDBANGLIMIT);
@@ -1009,17 +1009,19 @@ clobber_open(struct redir *f)
 
     /* If clobbering, just open. */
     if (isset(CLOBBER) || IS_CLOBBER_REDIR(f->type))
-	return open(unmeta(f->name), O_WRONLY | O_CREAT | O_TRUNC, 0666);
+	return open(unmeta(f->name),
+		O_WRONLY | O_CREAT | O_TRUNC | O_NOCTTY, 0666);
 
     /* If not clobbering, attempt to create file exclusively. */
-    if ((fd = open(unmeta(f->name), O_WRONLY | O_CREAT | O_EXCL, 0666)) >= 0)
+    if ((fd = open(unmeta(f->name),
+		   O_WRONLY | O_CREAT | O_EXCL | O_NOCTTY, 0666)) >= 0)
 	return fd;
 
     /* If that fails, we are still allowed to open non-regular files. *
      * Try opening, and if it's a regular file then close it again    *
      * because we weren't supposed to open it.                        */
     oerrno = errno;
-    if ((fd = open(unmeta(f->name), O_WRONLY)) != -1) {
+    if ((fd = open(unmeta(f->name), O_WRONLY | O_NOCTTY)) != -1) {
 	if(!fstat(fd, &buf) && !S_ISREG(buf.st_mode))
 	    return fd;
 	close(fd);
@@ -1653,9 +1655,10 @@ execcmd(Cmd cmd, int input, int output, int how, int last1)
 	    case READ:
 	    case READWRITE:
 		if (fn->type == READ)
-		    fil = open(unmeta(fn->name), O_RDONLY);
+		    fil = open(unmeta(fn->name), O_RDONLY | O_NOCTTY);
 		else
-		    fil = open(unmeta(fn->name), O_RDWR | O_CREAT, 0666);
+		    fil = open(unmeta(fn->name),
+			       O_RDWR | O_CREAT | O_NOCTTY, 0666);
 		if (fil == -1) {
 		    closemnodes(mfds);
 		    fixfds(save);
@@ -1695,7 +1698,8 @@ execcmd(Cmd cmd, int input, int output, int how, int last1)
 		if (IS_APPEND_REDIR(fn->type))
 		    fil = open(unmeta(fn->name),
 			       (unset(CLOBBER) && !IS_CLOBBER_REDIR(fn->type)) ?
-			       O_WRONLY | O_APPEND : O_WRONLY | O_APPEND | O_CREAT, 0666);
+			       O_WRONLY | O_APPEND | O_NOCTTY :
+			       O_WRONLY | O_APPEND | O_CREAT | O_NOCTTY, 0666);
 		else
 		    fil = clobber_open(fn);
 		if(fil != -1 && IS_ERROR_REDIR(fn->type))
@@ -2003,7 +2007,7 @@ entersubsh(int how, int cl, int fake)
 	    settrap(SIGQUIT, NULL);
 	    if (isatty(0)) {
 		close(0);
-		if (open("/dev/null", O_RDWR)) {
+		if (open("/dev/null", O_RDWR | O_NOCTTY)) {
 		    zerr("can't open /dev/null: %e", NULL, errno);
 		    _exit(1);
 		}
@@ -2143,11 +2147,11 @@ getherestr(struct redir *fn)
     unmetafy(t, &len);
     t[len++] = '\n';
     s = gettempname();
-    if (!s || (fd = open(s, O_CREAT | O_WRONLY | O_EXCL, 0600)) == -1)
+    if (!s || (fd = open(s, O_CREAT|O_WRONLY|O_EXCL|O_NOCTTY, 0600)) == -1)
 	return -1;
     write(fd, t, len);
     close(fd);
-    fd = open(s, O_RDONLY);
+    fd = open(s, O_RDONLY | O_NOCTTY);
     unlink(s);
     return fd;
 }
@@ -2181,7 +2185,7 @@ getoutput(char *cmd, int qt)
 	if (errflag)
 	    return NULL;
 	untokenize(s);
-	if ((stream = open(unmeta(s), O_RDONLY)) == -1) {
+	if ((stream = open(unmeta(s), O_RDONLY | O_NOCTTY)) == -1) {
 	    zerr("%e: %s", s, errno);
 	    return NULL;
 	}
@@ -2319,7 +2323,7 @@ getoutputfile(char *cmd)
 	addlinknode(jobtab[thisjob].filelist, nam);
     } LASTALLOC;
     child_block();
-    fd = open(nam, O_WRONLY | O_CREAT | O_EXCL, 0600); /* create the file */
+    fd = open(nam, O_WRONLY | O_CREAT | O_EXCL | O_NOCTTY, 0600);
 
     if (fd < 0 || (cmdoutpid = pid = zfork()) == -1) {
 	/* fork or open error */
@@ -2413,7 +2417,7 @@ getproc(char *cmd)
     }
 #ifndef PATH_DEV_FD
     closem(0);
-    fd = open(pnam, out ? O_WRONLY : O_RDONLY);
+    fd = open(pnam, out ? O_WRONLY | O_NOCTTY : O_RDONLY | O_NOCTTY);
     if (fd == -1) {
 	zerr("can't open %s: %e", pnam, errno);
 	_exit(1);
@@ -2768,7 +2772,7 @@ getfpfunc(char *s)
 	else
 	    strcpy(buf, s);
 	unmetafy(buf, NULL);
-	if (!access(buf, R_OK) && (fd = open(buf, O_RDONLY)) != -1) {
+	if (!access(buf, R_OK) && (fd = open(buf, O_RDONLY | O_NOCTTY)) != -1) {
 	    if ((len = lseek(fd, 0, 2)) != -1) {
 		lseek(fd, 0, 0);
 		d = (char *) zcalloc(len + 1);
