@@ -125,18 +125,9 @@ struct comp {
 #define GF_TOPLEV	2	/* outside (), so ~ ends main match */
 
 static char *pptr;		/* current place in string being matched */
-static Comp tail = 0;
+static Comp tail;
 static int first;		/* are leading dots special? */
 
-/**/
-static int
-exists(char *s)
-{
-    char *us = unmeta(s);
-
-    return access(us, F_OK) == 0 || readlink(us,NULL,0) == 0;
-}
-
 /* Add a component to pathbuf: This keeps track of how    *
  * far we are into a file name, since each path component *
  * must be matched separately.                            */
@@ -153,20 +144,31 @@ addpath(char *s)
     pathbuf[pathpos] = '\0';
 }
 
-/* return full path for s, which has path as *
- * already added to pathbuf                  */
+/* stat the filename s appended to pathbuf.  l should be true for lstat,    *
+ * false for stat.  If st is NULL, the file is only chechked for existance. *
+ * s == "" is treated as s == ".".  This is necessary since on most systems *
+ * foo/ can be used to reference a non-directory foo.  Returns nonzero if   *
+ * the file does not exists.                                                */
 
 /**/
-static char *
-getfullpath(char *s)
+static int
+statfullpath(const char *s, struct stat *st, int l)
 {
-    static char buf[PATH_MAX];
+    char buf[PATH_MAX];
 
-    DPUTS(strlen(s) + pathpos - pathbufcwd >= PATH_MAX,
-	  "BUG: getfullpath(): pathname too long");
+    DPUTS(strlen(s) + !*s + pathpos - pathbufcwd >= PATH_MAX,
+	  "BUG: statfullpath(): pathname too long");
     strcpy(buf, pathbuf + pathbufcwd);
     strcpy(buf + pathpos - pathbufcwd, s);
-    return buf;
+    if (!*s) {
+	buf[pathpos - pathbufcwd] = '.';
+	buf[pathpos - pathbufcwd + 1] = '\0';
+	l = 0;
+    }
+    unmetafy(buf, NULL);
+    if (!st)
+	return access(buf, F_OK) && (!l || readlink(buf, NULL, 0));
+    return l ? lstat(buf, st) : stat(buf, st);
 }
 
 /* add a match to the list */
@@ -176,70 +178,71 @@ static void
 insert(char *s, int checked)
 {
     struct stat buf, buf2, *bp;
+    char *news = s;
     int statted = 0;
 
     if (gf_listtypes || gf_markdirs) {
 	/* Add the type marker to the end of the filename */
+	mode_t mode;
 	checked = statted = 1;
-	if (gf_follow ? stat(unmetafy(getfullpath(s), NULL), &buf)
-	    : lstat(unmetafy(getfullpath(s), NULL), &buf))
+	if (statfullpath(s, &buf, 1))
 	    return;
-	if (gf_listtypes || S_ISDIR(buf.st_mode)) {
-	    char *t;
+	mode = buf.st_mode;
+	if (gf_follow) {
+	    if (!S_ISLNK(mode) || statfullpath(s, &buf2, 0))
+		memcpy(&buf2, &buf, sizeof(buf));
+	    statted = 2;
+	    mode = buf2.st_mode;
+	}
+	if (gf_listtypes || S_ISDIR(mode)) {
 	    int ll = strlen(s);
 
-	    t = (char *)ncalloc(ll + 2);
-	    strcpy(t, s);
-	    t[ll] = file_type(buf.st_mode);
-	    t[ll + 1] = '\0';
-	    s = t;
+	    news = (char *)ncalloc(ll + 2);
+	    strcpy(news, s);
+	    news[ll] = file_type(mode);
+	    news[ll + 1] = '\0';
 	}
     }
     if (qualct || qualorct) {
 	/* Go through the qualifiers, rejecting the file if appropriate */
 	struct qual *qo, *qn;
-	int t = 0;		/* reject file unless t is set */
 
-	if (!statted && lstat(unmetafy(getfullpath(s), NULL), &buf))
+	if (!statted && statfullpath(s, &buf, 1))
 	    return;
-	statted = 0;
-	for (qo = quals; qo && !t; qo = qo->or) {
-	    t = 1;
-	    for (qn = qo; t && qn && qn->func; qn = qn->next) {
-		range = qn->range;
-		amc = qn->amc;
-		units = qn->units;
-		if ((qn->sense & 2) && !statted) {
-		    /* If (sense & 2), we're following links */
-		    if (!S_ISLNK(buf.st_mode) ||
-			stat(unmetafy(getfullpath(s), NULL), &buf2))
-			memcpy(&buf2, &buf, sizeof(buf));
-		    statted = 1;
-		}
-		bp = (qn->sense & 2) ? &buf2 : &buf;
-		/* Reject the file if the function returned zero *
-		 * and the sense was positive (sense == 0), or   *
-		 * vice versa.                                   */
-		if (!(!!((qn->func) (bp, qn->data)) ^
-		      (qn->sense & 1))) {
-		    t = 0;
-		    break;
-		}
+	qo = quals;
+	for (qn = qo; qn && qn->func;) {
+	    range = qn->range;
+	    amc = qn->amc;
+	    units = qn->units;
+	    if ((qn->sense & 2) && statted != 2) {
+		/* If (sense & 2), we're following links */
+		if (!S_ISLNK(buf.st_mode) || statfullpath(s, &buf2, 0))
+		    memcpy(&buf2, &buf, sizeof(buf));
+		statted = 2;
 	    }
+	    bp = (qn->sense & 2) ? &buf2 : &buf;
+	    /* Reject the file if the function returned zero *
+	     * and the sense was positive (sense&1 == 0), or *
+	     * vice versa.                                   */
+	    if ((!((qn->func) (bp, qn->data)) ^ qn->sense) & 1) {
+		/* Try next alternative, or return if there are no more */
+		if (!(qo = qo->or))
+		    return;
+		qn = qo;
+		continue;
+	    }
+	    qn = qn->next;
 	}
-	if (!t)
-	    return;
-    } else if (!checked && !exists(getfullpath(s)))
+    } else if (!checked && statfullpath(s, NULL, 1))
 	return;
 
-    s = dyncat(pathbuf, s);
+    news = dyncat(pathbuf, news);
     if (colonmod) {
 	/* Handle the remainder of the qualifer:  e.g. (:r:s/foo/bar/). */
-	char *cm2 = colonmod;
-
-	modify(&s, &cm2);
+	s = colonmod;
+	modify(&news, &s);
     }
-    *matchptr++ = s;
+    *matchptr++ = news;
     if (++matchct == matchsz) {
 	matchbuf = (char **)realloc((char *)matchbuf,
 				    sizeof(char **) * (matchsz *= 2));
@@ -264,8 +267,6 @@ haswilds(char *str)
     if (str[0] == '%' && str[1] == Quest)
 	str[1] = '?';
 
-    if(*str == Tilde)
-	str++;
     for (; *str; str++) {
 	switch (*str) {
 	    case Inpar:
@@ -277,7 +278,6 @@ haswilds(char *str)
 		return 1;
 	    case Pound:
 	    case Hat:
-	    case Tilde:
 		if (isset(EXTENDEDGLOB))
 		    return 1;
 		break;
@@ -316,7 +316,7 @@ scanner(Complist q)
 	/* It's a straight string to the end of the path section. */
 	int l = strlen(c->str);
 
-	if (l + pathpos - pathbufcwd >= PATH_MAX) {
+	if (l + !l + pathpos - pathbufcwd >= PATH_MAX) {
 	    int err;
 
 	    if (l >= PATH_MAX)
@@ -336,7 +336,7 @@ scanner(Complist q)
 
 	    if (!errflag && !(q->closure && !strcmp(c->str, "."))) {
 		addpath(c->str);
-		if (!closure || exists(pathbuf))
+		if (!closure || statfullpath("", NULL, 1))
 		    scanner((q->closure) ? q : q->next);
 		pathbuf[pathpos = oppos] = '\0';
 	    }
@@ -392,9 +392,7 @@ scanner(Complist q)
 			/* if matching multiple directories */
 			struct stat buf;
 
-			if ((q->follow ?
-			     stat(unmeta(getfullpath(fn)), &buf) :
-			     lstat(unmeta(getfullpath(fn)), &buf)) == -1) {
+			if (statfullpath(fn, &buf, !q->follow)) {
 			    if (errno != ENOENT && errno != EINTR &&
 				errno != ENOTDIR && !errflag) {
 				zerr("%e: %s", fn, errno);
@@ -803,6 +801,7 @@ parsepat(char *str)
 {
     mode = 0;			/* path components present */
     pptr = str;
+    tail = NULL;
     return parsecomplist();
 }
 
@@ -2435,6 +2434,7 @@ parsereg(char *str)
     remnulargs(str);
     mode = 1;			/* no path components */
     pptr = str;
+    tail = NULL;
     return parsecompsw(GF_TOPLEV);
 }
 
