@@ -1,6 +1,4 @@
 /*
- * $Id: hashtable.c,v 3.1.2.3 1997/06/01 06:13:15 hzoli Exp $
- *
  * hashtable.c - hash tables
  *
  * This file is part of zsh, the Z shell.
@@ -29,7 +27,7 @@
  *
  */
 
-#include <config.h>
+#include "../config.h"
 
 #ifdef ZSH_HASH_DEBUG
 # define HASHTABLE_DEBUG_MEMBERS \
@@ -47,7 +45,7 @@
 
 typedef struct scanstatus *ScanStatus;
 
-#include "zsh.h"
+#include "zsh.mdh"
 #include "hashtable.pro"
 
 /* Structure for recording status of a hashtable scan in progress.  When a *
@@ -555,6 +553,16 @@ bin_hashinfo(char *nam, char **args, char *ops, int func)
 /* Command Hash Table Functions */
 /********************************/
 
+/* hash table containing external commands */
+ 
+/**/
+HashTable cmdnamtab;
+ 
+/* how far we've hashed the PATH so far */
+ 
+/**/
+char **pathchecked;
+ 
 /* Create a new command hash table */
  
 /**/
@@ -696,6 +704,11 @@ printcmdnamnode(HashNode hn, int printflags)
 /* Shell Function Hash Table Functions */
 /***************************************/
 
+/* hash table containing the shell functions */
+
+/**/
+HashTable shfunctab;
+
 /**/
 void
 createshfunctable(void)
@@ -825,74 +838,45 @@ printshfuncnode(HashNode hn, int printflags)
     zsfree(t);
 }
 
-/****************************************/
-/* Builtin Command Hash Table Functions */
-/****************************************/
-
-/**/
-void
-createbuiltintable(void)
-{
-    Builtin bn;
-
-    builtintab = newhashtable(85, "builtintab", NULL);
-
-    builtintab->hash        = hasher;
-    builtintab->emptytable  = NULL;
-    builtintab->filltable   = NULL;
-    builtintab->addnode     = addhashnode;
-    builtintab->getnode     = gethashnode;
-    builtintab->getnode2    = gethashnode2;
-    builtintab->removenode  = removehashnode;
-    builtintab->disablenode = disablehashnode;
-    builtintab->enablenode  = enablehashnode;
-    builtintab->freenode    = freebuiltinnode;
-    builtintab->printnode   = printbuiltinnode;
-
-    for (bn = builtins; bn->nam; bn++)
-	builtintab->addnode(builtintab, bn->nam, bn);
-}
-
-/* Print a builtin */
-
-/**/
-static void
-printbuiltinnode(HashNode hn, int printflags)
-{
-    Builtin bn = (Builtin) hn;
-
-    if (printflags & PRINT_WHENCE_CSH) {
-	printf("%s: shell built-in command\n", bn->nam);
-	return;
-    }
-
-    if (printflags & PRINT_WHENCE_VERBOSE) {
-	printf("%s is a shell builtin\n", bn->nam);
-	return;
-    }
-
-    /* default is name only */
-    printf("%s\n", bn->nam);
-}
-
-/**/
-static void
-freebuiltinnode(HashNode hn)
-{
-    Builtin bn = (Builtin) hn;
- 
-    zsfree(bn->nam);
-    zsfree(bn->optstr);
-    zsfree(bn->defopts);
-    zfree(bn, sizeof(struct builtin));
-}
-
-
-
 /**************************************/
 /* Reserved Word Hash Table Functions */
 /**************************************/
 
+/* Nodes for reserved word hash table */
+
+static struct reswd reswds[] = {
+    {NULL, "!", 0, BANG},
+    {NULL, "[[", 0, DINBRACK},
+    {NULL, "{", 0, INBRACE},
+    {NULL, "}", 0, OUTBRACE},
+    {NULL, "case", 0, CASE},
+    {NULL, "coproc", 0, COPROC},
+    {NULL, "do", 0, DO},
+    {NULL, "done", 0, DONE},
+    {NULL, "elif", 0, ELIF},
+    {NULL, "else", 0, ELSE},
+    {NULL, "end", 0, ZEND},
+    {NULL, "esac", 0, ESAC},
+    {NULL, "fi", 0, FI},
+    {NULL, "for", 0, FOR},
+    {NULL, "foreach", 0, FOREACH},
+    {NULL, "function", 0, FUNC},
+    {NULL, "if", 0, IF},
+    {NULL, "nocorrect", 0, NOCORRECT},
+    {NULL, "repeat", 0, REPEAT},
+    {NULL, "select", 0, SELECT},
+    {NULL, "then", 0, THEN},
+    {NULL, "time", 0, TIME},
+    {NULL, "until", 0, UNTIL},
+    {NULL, "while", 0, WHILE},
+    {NULL, NULL}
+};
+
+/* hash table containing the reserved words */
+
+/**/
+HashTable reswdtab;
+
 /* Build the hash table containing zsh's reserved words. */
 
 /**/
@@ -945,6 +929,11 @@ printreswdnode(HashNode hn, int printflags)
 /* Aliases Hash Table Functions */
 /********************************/
 
+/* hash table containing the aliases */
+ 
+/**/
+HashTable aliastab;
+ 
 /* Create new hash table for aliases */
 
 /**/
@@ -1146,8 +1135,14 @@ printparamnode(HashNode hn, int printflags)
 /* Named Directory Hash Table Functions */
 /****************************************/
 
+/* hash table containing named directories */
+
+/**/
+HashTable nameddirtab;
+ 
 /* != 0 if all the usernames have already been *
  * added to the named directory hash table.    */
+
 static int allusersadded;
 
 /* Create new hash table for named directories */
