@@ -1568,9 +1568,11 @@ savehistfile(char *s, int err, int app)
     if (ev < firsthist())
 	ev = firsthist();
     if (app & 1)
-	out = fdopen(open(unmeta(s), O_CREAT | O_WRONLY | O_APPEND, 0600), "a");
+	out = fdopen(open(unmeta(s),
+		     O_CREAT | O_WRONLY | O_APPEND | O_NOCTTY, 0600), "a");
     else
-	out = fdopen(open(unmeta(s), O_CREAT | O_WRONLY | O_TRUNC, 0600), "w");
+	out = fdopen(open(unmeta(s),
+		     O_CREAT | O_WRONLY | O_TRUNC | O_NOCTTY, 0600), "w");
     if (out) {
 	for (; ev <= curhist - !!(histactive & HA_ACTIVE); ev++) {
 	    ent = gethistent(ev);
@@ -1622,7 +1624,8 @@ savehistfile(char *s, int err, int app)
 		histnum++;
 	    }
 	    fclose(out);
-	    if ((out = fdopen(open(unmeta(s), O_WRONLY | O_TRUNC, 0600), "w"))) {
+	    if ((out = fdopen(open(unmeta(s),
+			    O_WRONLY | O_TRUNC | O_NOCTTY, 0600), "w"))) {
 		if (histnum < savehist)
 		    for (i = 0; i < histnum; i++)
 			fprintf(out, "%s", store[i]);
