@@ -228,7 +228,7 @@ parseargs(char **argv)
 	if (unset(SHINSTDIN)) {
 	    argzero = *argv;
 	    if (!cmd)
-		SHIN = movefd(open(unmeta(argzero), O_RDONLY));
+		SHIN = movefd(open(unmeta(argzero), O_RDONLY | O_NOCTTY));
 	    if (SHIN == -1) {
 		zerr("can't open input file: %s", argzero, 0);
 		exit(1);
@@ -293,9 +293,10 @@ init_io(void)
     if (isatty(0)) {
 	zsfree(ttystrname);
 	if ((ttystrname = ztrdup(ttyname(0))))
-	    SHTTY = movefd(open(ttystrname, O_RDWR));
+	    SHTTY = movefd(open(ttystrname, O_RDWR | O_NOCTTY));
     }
-    if (SHTTY == -1 && (SHTTY = movefd(open("/dev/tty", O_RDWR))) != -1) {
+    if (SHTTY == -1 &&
+	(SHTTY = movefd(open("/dev/tty", O_RDWR | O_NOCTTY))) != -1) {
 	zsfree(ttystrname);
 	ttystrname = ztrdup("/dev/tty");
     }
@@ -737,7 +738,7 @@ init_misc(void)
     if (cmd) {
 	if (SHIN >= 10)
 	    fclose(bshin);
-	SHIN = movefd(open("/dev/null", O_RDONLY));
+	SHIN = movefd(open("/dev/null", O_RDONLY | O_NOCTTY));
 	bshin = fdopen(SHIN, "r");
 	execstring(cmd, 0, 1);
 	stopmsg = 1;
@@ -759,7 +760,7 @@ source(char *s)
     FILE *obshin;
     char *old_scriptname = scriptname;
 
-    if (!s || (tempfd = movefd(open(unmeta(s), O_RDONLY))) == -1) {
+    if (!s || (tempfd = movefd(open(unmeta(s), O_RDONLY | O_NOCTTY))) == -1) {
 	return 1;
     }
 
