@@ -1043,13 +1043,20 @@ gettokstr(int c, int sub)
 	    if (lexstop)
 		goto brk;
 	    break;
-	case LX2_QUOTE:
+	case LX2_QUOTE: {
+	    int strquote = (len && bptr[-1] == String);
+
 	    add(Snull);
 	    cmdpush(CS_QUOTE);
 	    for (;;) {
 		STOPHIST
 		while ((c = hgetc()) != '\'' && !lexstop) {
-		    if (!sub && isset(CSHJUNKIEQUOTES) && c == '\n') {
+		    if (strquote && c == '\\') {
+			add(c);
+			c = hgetc();
+			if (lexstop)
+			    break;
+		    } else if (!sub && isset(CSHJUNKIEQUOTES) && c == '\n') {
 			if (bptr[-1] == '\\')
 			    bptr--, len--;
 			else
@@ -1074,6 +1081,7 @@ gettokstr(int c, int sub)
 	    lexstop = 0;
 	    c = Snull;
 	    break;
+	}
 	case LX2_DQUOTE:
 	    add(Dnull);
 	    cmdpush(CS_DQUOTE);
@@ -1452,13 +1460,20 @@ skipcomm(void)
 	    add(c);
 	    c = hgetc();
 	    break;
-	case '\'':
+	case '\'': {
+	    int strquote = bptr[-1] == '$';
 	    add(c);
 	    STOPHIST
-	    while ((c = hgetc()) != '\'' && !lexstop)
+	    while ((c = hgetc()) != '\'' && !lexstop) {
+		if (c == '\\' && strquote) {
+		    add(c);
+		    c = hgetc();
+		}
 		add(c);
+	    }
 	    ALLOWHIST
 	    break;
+	}
 	case '\"':
 	    add(c);
 	    while ((c = hgetc()) != '\"' && !lexstop)
