@@ -1258,8 +1258,9 @@ setaparam(char *s, char **val)
 	    createparam(t, PM_ARRAY);
 	else if (PM_TYPE(v->pm->flags) != PM_ARRAY &&
 		 !(v->pm->flags & PM_SPECIAL)) {
+	    int uniq = v->pm->flags & PM_UNIQUE;
 	    unsetparam(t);
-	    createparam(t, PM_ARRAY);
+	    createparam(t, PM_ARRAY | uniq);
 	    v = NULL;
 	}
     }
@@ -1334,7 +1335,8 @@ unsetparam_pm(Param pm, int altflag, int exp)
     /* remove it under its alternate name if necessary */
     if (pm->ename && !altflag) {
 	altpm = (Param) paramtab->getnode(paramtab, pm->ename);
-	unsetparam_pm(altpm, 1, exp);
+	if (altpm)
+	    unsetparam_pm(altpm, 1, exp);
     }
 
     if (locallevel >= pm->level && (pm->flags & PM_SPECIAL))
