@@ -115,6 +115,9 @@ stringsubst(LinkList list, LinkNode node, int ssub)
 		str = arithsubst(str + 2, &str3, str2);
 		setdata(node, (void *) str3);
 		continue;
+	    } else if (str[1] == Snull) {
+		str = getkeystring(str, NULL, 4, NULL);
+		continue;
 	    } else {
 		node = paramsubst(list, node, &str, qt, ssub);
 		if (errflag || !node)
