@@ -28,9 +28,13 @@
  */
 
 #ifdef __hpux
-#define _INCLUDE_POSIX_SOURCE
-#define _INCLUDE_XOPEN_SOURCE
-#define _INCLUDE_HPUX_SOURCE
+# define _INCLUDE_POSIX_SOURCE 1
+# define _INCLUDE_XOPEN_SOURCE 1
+# define _INCLUDE_HPUX_SOURCE 1
+#endif
+
+#ifdef sinix
+# define _XPG_IV 1
 #endif
 
 /* NeXT has half-implemented POSIX support *
@@ -66,18 +70,15 @@ char *alloca _((size_t));
 # endif
 #endif
 
-#ifdef HAVE_VARIABLE_LENGTH_ARRAYS
-# define VARARR(X,Y,Z)	X (Y)[Z]
-#else
-# define VARARR(X,Y,Z)	X *(Y) = (X *) alloca(sizeof(X) * (Z))
-#endif
-
 #ifdef HAVE_LIBC_H     /* NeXT */
 # include <libc.h>
 #endif
 
-#ifdef HAVE_UNISTD_H
+#ifdef HAVE_SYS_TYPES_H
 # include <sys/types.h>
+#endif
+
+#ifdef HAVE_UNISTD_H
 # include <unistd.h>
 #endif
 
@@ -168,6 +169,12 @@ struct timezone {
 # include <limits.h>
 #endif
 
+#ifdef HAVE_VARIABLE_LENGTH_ARRAYS
+# define VARARR(X,Y,Z)	X (Y)[Z]
+#else
+# define VARARR(X,Y,Z)	X *(Y) = (X *) alloca(sizeof(X) * (Z))
+#endif
+
 /* we should be getting this value from pathconf(_PC_PATH_MAX) */
 /* but this is too much trouble                                */
 #ifndef PATH_MAX
@@ -205,13 +212,13 @@ struct timezone {
 #ifdef HAVE_SYS_WAIT_H
 # include <sys/wait.h>
 #else
-#undef WIFEXITED
-#undef WEXITSTATUS
-#undef WIFSIGNALED
-#undef WTERMSIG
-#undef WCOREDUMP
-#undef WIFSTOPPED
-#undef WSTOPSIG
+# undef WIFEXITED
+# undef WEXITSTATUS
+# undef WIFSIGNALED
+# undef WTERMSIG
+# undef WCOREDUMP
+# undef WIFSTOPPED
+# undef WSTOPSIG
 #endif
 
 /* missing macros for wait/waitpid/wait3 */
@@ -272,7 +279,7 @@ struct timezone {
 #endif   /* HAVE_TERMIOS_H */
 
 #ifdef HAVE_TERMCAP_H
-#include <termcap.h>
+# include <termcap.h>
 #endif
 
 #if defined(GWINSZ_IN_SYS_IOCTL) || defined(CLOBBERS_TYPEAHEAD)
@@ -291,42 +298,7 @@ struct timezone {
 # include <sys/utsname.h>
 #endif
 
-#ifdef HAVE_UTMPX_H
-# include <utmpx.h>
-# define STRUCT_UTMP struct utmpx
-# define ut_time ut_xtime
-#else
-# include <utmp.h>
-# define STRUCT_UTMP struct utmp
-#endif
- 
-#if !defined (UTMP_FILE) && defined (_PATH_UTMP)        /* 4.4BSD.  */
-# define UTMP_FILE _PATH_UTMP
-#endif
-
-#if !defined (WTMP_FILE) && defined (_PATH_WTMP)
-# define WTMP_FILE _PATH_WTMP
-#endif
- 
-#ifdef UTMPX_FILE                                /* Solaris, SysVr4 */
-# undef  UTMP_FILE
-# define UTMP_FILE UTMPX_FILE
-#endif
-
-#ifdef WTMPX_FILE                                /* Solaris. SysVr4 */
-# undef  WTMP_FILE
-# define WTMP_FILE WTMPX_FILE
-#endif
- 
-#ifndef UTMP_FILE                                /* use value found by configure */
-# define UTMP_FILE UTMP_FILE_CONFIG
-#endif
-
-#ifndef WTMP_FILE                                /* use value found by configure */
-# define WTMP_FILE WTMP_FILE_CONFIG
-#endif
-
-#ifdef HAVE_UT_HOST
+#if defined(HAVE_UTMPX_UT_HOST) || defined(HAVE_UTMP_UT_HOST)
 # define DEFAULT_WATCHFMT "%n has %a %l from %m."
 #else
 # define DEFAULT_WATCHFMT "%n has %a %l."
@@ -600,12 +572,15 @@ struct timezone {
 # define offsetof(TYPE, MEM) ((char *)&((TYPE *)0)->MEM - (char *)(TYPE *)0)
 #endif
 
-extern char **environ;     /* environment variable list */
+extern char **environ;
 
 /* These variables are sometimes defined in, *
  * and needed by, the termcap library.       */
- 
 #if MUST_DEFINE_OSPEED
 extern char PC, *BC, *UP;
 extern short ospeed;
 #endif
+
+#ifndef O_NOCTTY
+# define O_NOCTTY 0
+#endif
