@@ -3209,12 +3209,18 @@ dquotedzputs(char const *s, FILE *stream)
 char *
 getkeystring(char *s, int *len, int fromwhere, int *misc)
 {
-    char *buf = halloc(strlen(s) + 1);
-    char *t = buf, *u = NULL;
+    char *buf;
+    char *t, *u = NULL;
     char svchar = '\0';
     int meta = 0, control = 0;
 
-    for (; *s; s++) {
+    if (fromwhere != 4)
+	buf = halloc(strlen(s) + 1);
+    else {
+	buf = s;
+	s += 2;
+    }
+    for (t = buf; *s; s++) {
 	if (*s == '\\' && s[1]) {
 	    switch (*++s) {
 	    case 'a':
@@ -3302,6 +3308,9 @@ getkeystring(char *s, int *len, int fromwhere, int *misc)
 		}
 		break;
 	    }
+	} else if (fromwhere == 4 && *s == Snull) {
+	    for (u = t; (*u++ = *s++););
+	    return t + 1;
 	} else if (*s == '^' && fromwhere == 2) {
 	    control = 1;
 	    continue;
@@ -3324,7 +3333,13 @@ getkeystring(char *s, int *len, int fromwhere, int *misc)
 	    t[-1] |= 0x80;
 	    meta = 0;
 	}
+	if (fromwhere == 4 && imeta(t[-1])) {
+	    *t = t[-1] ^ 32;
+	    t[-1] = Meta;
+	    t++;
+	}
     }
+    DPUTS(fromwhere == 4, "BUG: unterminated $' substitution");
     *t = '\0';
     *len = t - buf;
     return buf;
@@ -3437,9 +3452,9 @@ lchdir(char const *path, struct dirsav *d, int hard)
 #endif
 	level = -1;
 #ifdef HAVE_FCHDIR
-	if (d->dirfd < 0 && (d->dirfd = open(".", O_RDONLY)) < 0 &&
+	if (d->dirfd < 0 && (d->dirfd = open(".", O_RDONLY | O_NOCTTY)) < 0 &&
 	    zgetdir(d) && *d->dirname != '/')
-	    d->dirfd = open("..", O_RDONLY);
+	    d->dirfd = open("..", O_RDONLY | O_NOCTTY);
 #else
 	if (!d->dirname)
 	    zgetdir(d);
